import 'package:flutter/widgets.dart';
import 'package:microfrontends/src/context.dart';
import 'package:microfrontends/src/query.dart';

typedef EffectCallback<T> = void Function(BuildContext context, T value);

class Effect<T> {
  static final _context = Context<BuildContext, Effect>();

  static void dispatch<T>(BuildContext context, T value) {
    final effect = _context[context] as Effect<T>;
    effect._callback(context, value);
  }

  static deactivate(BuildContext context) {
    _context.remove(context);
  }

  static Effect<T?> nullable<T>(EffectCallback<T?> callback) {
    return Effect<T?>(callback);
  }

  final EffectCallback<T> _callback;
  late Query<T> _query;

  Effect(this._callback, [Object? queryParams]) {
    _query = Query<T>(queryParams);
  }

  void whenType<T>() {
    _query = _query.withUpdateCondition((state) {
      return state is T;
    });
  }

  void bind(BuildContext context, {leading = false}) {
    _context[context] = this;
    final v = _query.subscribe(context, offstage: true);

    if (null is T) {
      final cb = _callback as EffectCallback<T?>;
      if (leading) {
        cb.call(context, v);
      }
    } else if (leading) {
      _callback.call(context, v!);
    }
  }
}
