import 'dart:async';

import 'package:flutter/widgets.dart';

import 'resolver.dart';
import 'extensions.dart';

abstract class Mutation<T> {
  final T payload;

  Mutation(this.payload);

  void exec<K>(BuildContext context) {
    final r = context.getMutationResolverOfType<K>();
    final container = r.getContainer(context);
    final newState = r.resolve(container.stateNotifier, this);
    container.stateNotifier.value = newState;
    return newState;
  }
}

class MutationResolver<R, A extends Mutation, S> extends Resolver<S, R, A> {
  MutationResolver(
      ResolverController<S> ctrl, ResolverCallback<R, S, A> callback)
      : super(ctrl, callback);
}

class SetState<T> extends Mutation<T> {
  Type get key => T;
  SetState(T payload) : super(payload);
}
