import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

import 'context.dart';
import 'mutation.dart';
import 'query.dart';
import 'resolver.dart';
import 'extensions.dart';
import 'state_container_context.dart';

abstract class StateContainer<T> {
  late final ValueNotifier<T> stateNotifier = ValueNotifier(initialState);
  late final resolverController = ResolverController<T>(stateNotifier);

  final queryResolvers = Context<Type, QueryResolver>({});
  final mutationResolvers = Context<Type, MutationResolver>();

  T get state {
    return stateNotifier.value;
  }

  set state(T state) {
    stateNotifier.value = state;
  }

  Type get stateType => T;

  T get initialState;

  static StateContainer<T> of<T>(BuildContext context) =>
      context.getContainerOfType<T>();

  Stream<T> init(StateContainerContext context) async* {}

  @mustCallSuper
  queries(StateContainerContext context) {
    query((ctrl, value, arg) => value);
  }

  @mustCallSuper
  mutations(StateContainerContext context) {
    mutation<SetState<T>>((ctrl, value, arg) => arg.payload);
  }

  @mustCallSuper
  shutDown() {}

  void query<K>(ResolverCallback<K, T, Object?> callback) {
    final queryResolver = QueryResolver<T, K>(resolverController, callback);
    queryResolvers[queryResolver.keyType] = queryResolver;
  }

  void mutation<K extends Mutation>(
      ResolverCallback<FutureOr<void>, T, K> callback) {
    final mutationResolver =
        MutationResolver<FutureOr<void>, K, T>(resolverController, callback);
    mutationResolvers[K] = mutationResolver;
  }
}
