import 'package:flutter/widgets.dart';

import 'chunked_state_container.dart';
import 'state_container.dart';
import 'static_container.dart';

typedef StateChunkSetter = void Function<T>(T value);

abstract class StateContainerWidget extends Widget {
  const StateContainerWidget({Key? key}) : super(key: key);

  void initState(StateChunkSetter state);

  Stream init() async* {}

  @override
  Element createElement() {
    return StateContainerWidgetElement(this);
  }

  Widget build(BuildContext context);
}

class StateContainerWidgetElement extends ComponentElement {
  StateContainerWidgetElement(Widget widget) : super(widget);

  StateContainerWidget get widget => super.widget as StateContainerWidget;
  StateContainer? container;

  List<StateChunk> chunks = [];

  void state<T>(T value) {
    chunks.add(StateChunk<T>(value));
  }

  void setState<T>(BuildContext context, T value) {
    context.setState(value);
  }

  @override
  void mount(Element? parent, Object? newSlot) {
    widget.initState(state);
    final initStream = widget.init();
    container = ChunkedStateContainer(chunks, initStream);

    super.mount(parent, newSlot);
  }

  @override
  Widget build() {
    return StaticContainer(
      container: container!,
      child: Builder(builder: (context) => widget.build(context)),
    );
  }
}
