import 'package:flutter/widgets.dart';

import 'query.dart';

typedef QueryResultBuilder<T> = Widget Function(
  BuildContext context,
  T data,
  Widget? child,
);

class QueryBuilder<T> extends Widget {
  final Widget? child;
  final QueryResultBuilder<T>? builder;
  final Object? params;
  final QueryResultBuilder<T?>? nullableBuilder;

  static QueryBuilder<T?> nullable<T>({
    Key? key,
    required QueryResultBuilder<T?> builder,
    Widget? child,
    Object? params,
  }) {
    return QueryBuilder<T?>(
      key: key,
      nullableBuilder: builder,
      child: child,
      params: params,
    );
  }

  QueryBuilder({
    Key? key,
    this.builder,
    this.nullableBuilder,
    this.child,
    this.params,
  })  : assert(
          null is! T || nullableBuilder != null,
          'Use QueryBuilder.nullable<${T.toString().replaceAll('?', '')}>() instead of QueryBuilder<$T>',
        ),
        super(key: key);

  @override
  Element createElement() {
    return QueryBuilderElement<T>(this);
  }
}

class QueryBuilderElement<T> extends ComponentElement {
  QueryBuilderElement(Widget widget) : super(widget);

  @override
  QueryBuilder<T> get widget => super.widget as QueryBuilder<T>;

  late final query = Query<T>(widget.params);

  @override
  Widget build() {
    final d = query.subscribe(this);

    if (null is T) {
      assert(
        widget.nullableBuilder != null,
        'Received a null $T. '
        'Use QueryBuilder.nullable<$T> instead of QueryBuilder<$T>',
      );

      return widget.nullableBuilder!(this, d, widget.child);
    }

    return widget.builder!(this, d!, widget.child);
  }

  @override
  void deactivate() {
    query.unsubscribe(this);
    super.deactivate();
  }
}
